package com.codigoparallevar.minicards.toolbox;

import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import androidx.appcompat.app.AlertDialog;
import android.util.Log;

import com.codigoparallevar.minicards.CanvasView;
import com.codigoparallevar.minicards.PartInstantiator;
import com.codigoparallevar.minicards.parts.buttons.RoundButton;
import com.codigoparallevar.minicards.parts.logic.Ticker;
import com.codigoparallevar.minicards.parts.logic.Toggle;
import com.codigoparallevar.minicards.parts.samples.ColorBox;
import com.codigoparallevar.minicards.parts.strings.ConvertToString;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.functional.Tuple2;
import com.programaker.api.data.ProgramakerBridgeInfo;
import com.programaker.api.data.ProgramakerCustomBlock;
import com.programaker.api.data.api_results.ProgramakerBridgeCustomBlockResult;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

public class PartsHolder {
    private final Context context;

    private final static List<Tuple2<String, PartInstantiator>> BuiltInParts =
            new Vector<Tuple2<String, PartInstantiator>>(){{
                add(new Tuple2<>("Round button", RoundButton.getInstantiator()));
                add(new Tuple2<>("Ticker", Ticker.getInstantiator()));
                add(new Tuple2<>("Red/Green box", ColorBox.getInstantiator()));
                add(new Tuple2<>("Toggle", Toggle.getInstantiator()));
                add(new Tuple2<>("ToString", ConvertToString.getInstantiator()));
    }};

    private final static List<PartCategory> Categories = new Vector<PartCategory>() {{
        add(new PartCategory("Testing", BuiltInParts));
    }};
    private Map<String, ProgramakerBridgeInfo> bridgeInfoMap;

    public PartsHolder(Context context) {
        this.context = context;
    }

    public void openAddPartModal(final CanvasView canvasView) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        Map<String, PartCategory> categoryOptions = getPartCategories();
        String[] categoryNames = categoryOptions.keySet().toArray(new String[0]);

        builder.setTitle("Choose part category")
                .setItems(categoryNames, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int whichCategory) {
                        if ((whichCategory >= 0) && (whichCategory < categoryNames.length)){
                            Map<String, PartInstantiator> parts = getPartTypes(categoryOptions.get(categoryNames[whichCategory]));
                            String[] partNames = parts.keySet().toArray(new String[0]);

                            builder.setTitle("Choose part type")
                                    .setItems(partNames, (dialog1, whichPart) -> {
                                        if ((whichPart >= 0) && (whichPart < partNames.length)) {
                                            String partName = partNames[whichPart];

                                            Log.d("Minicards partsHolder",
                                                    "Spawning " + partName);

                                            PartInstantiator instantiator = parts.get(partName);
                                            PartsHolder.this.runInstantiator(instantiator, canvasView);
                                        }
                                    });

                            Dialog dialog2 = builder.create();
                            dialog2.show();
                        }
                    }
                });

        Dialog dialog = builder.create();
        dialog.show();
    }

    private void runInstantiator(PartInstantiator instantiator, CanvasView canvasView) {
        Part part = instantiator.build(canvasView);
        canvasView.addPart(part);
    }

    private static Map<String,PartCategory> getPartCategories() {
        Map<String, PartCategory> partTypes = new LinkedHashMap<>(Categories.size());
        for (int i = 0; i < Categories.size(); i++){
            partTypes.put(Categories.get(i).getName(), Categories.get(i));
        }

        return partTypes;
    }

    private static Map<String, PartInstantiator> getPartTypes(PartCategory categoryOption) {
        HashMap<String, PartInstantiator> partMap = new LinkedHashMap<>();

        for (Tuple2<String, PartInstantiator> part : categoryOption.getParts()) {
            partMap.put(part.item1, part.item2);
        }

        return partMap;
    }

    public void addCustomBlocks(List<ProgramakerBridgeInfo> bridgeInfo, List<ProgramakerBridgeCustomBlockResult> customBlocks) {
        Map<String, ProgramakerBridgeInfo> bridgeInfoMap = new LinkedHashMap<>();
        for (ProgramakerBridgeInfo info : bridgeInfo) {
            bridgeInfoMap.put(info.getId(), info);
        }

        this.bridgeInfoMap = bridgeInfoMap;

        for (ProgramakerBridgeCustomBlockResult res : customBlocks) {

            List<Tuple2<String, PartInstantiator>> parts = new LinkedList<>();
            for (ProgramakerCustomBlock block : res.getBlocks()) {
                parts.add(new Tuple2<>(block.getMessage(), new ProgramakerCustomBlockInstantiator(block)));
            }

            if (bridgeInfoMap.containsKey(res.getBridge_id())) {
                Categories.add(new PartCategory(bridgeInfoMap.get(res.getBridge_id()).getName(), parts));
            }
            else {
                Categories.add(new PartCategory("Bridge ID="+res.getBridge_id(), parts));
            }
        }
    }
}
