package com.codigoparallevar.minicards.types;

import com.codigoparallevar.minicards.types.connectors.output.OutputConnector;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.LinkedHashMap;
import java.util.Map;

public class PartConnection {
    private static final String INPUT_CONNECTOR_ID_KEY = "input_connector_id";
    private static final String INPUT_PART_ID_KEY = "input_part_id";
    public final OutputConnector outputConnector;
    public final String inputPartId;
    public final String inputConnectorId;


    public PartConnection(OutputConnector outputConnector, String inputPartId,
                          String inputConnectorId) {
        this.outputConnector = outputConnector;
        this.inputPartId = inputPartId;
        this.inputConnectorId = inputConnectorId;
    }

    public static PartConnection deserialize(OutputConnector outputConnector, JSONObject metadata) throws JSONException {
        String inputConnectorId = metadata.getString(PartConnection.INPUT_CONNECTOR_ID_KEY);
        String inputPartId = metadata.getString(PartConnection.INPUT_PART_ID_KEY);

        return new PartConnection(outputConnector, inputPartId, inputConnectorId);
    }

    public static Map<String, String> serialize(final String inputConnectorId, final String inputPartId) {

        Map<String, String> serialized = new LinkedHashMap<>(2);
        serialized.put(PartConnection.INPUT_CONNECTOR_ID_KEY, inputConnectorId);
        serialized.put(PartConnection.INPUT_PART_ID_KEY, inputPartId);

        return serialized;
    }

    public static JSONObject serializeToJson(final String inputConnectorId, final String inputPartId) {

        JSONObject serialized = new JSONObject();
        try {
            serialized.put(PartConnection.INPUT_CONNECTOR_ID_KEY, inputConnectorId);
            serialized.put(PartConnection.INPUT_PART_ID_KEY, inputPartId);
        } catch (JSONException e) {
            e.printStackTrace();
        }

        return serialized;
    }
}
