package com.programaker.bridge

import android.util.Log
import okhttp3.*
import okio.ByteString
import java.util.concurrent.TimeUnit


class ProgramakerBridge(
        val bridge_id: String,
        val user_id: String,
        val config: ProgramakerBridgeConfiguration,
        val onComplete: Runnable
) : WebSocketListener() {
    private val PING_PERIOD_MILLIS: Long = 15000
    private val apiRoot: String = "wss://programaker.com/api"
    private val LogTag = "ProgramakerBridge"

    fun run() {
        val client: OkHttpClient = OkHttpClient.Builder()
                .pingInterval(PING_PERIOD_MILLIS, TimeUnit.MILLISECONDS)
                .readTimeout(0, TimeUnit.MILLISECONDS)
                .build()

        val request: Request = Request.Builder()
                .url(getBridgeControlUrl())
                .build()
        client.newWebSocket(request, this)

        // Trigger shutdown of the dispatcher's executor so this process can exit cleanly.
        client.dispatcher.executorService.shutdown()
    }


    override fun onOpen(webSocket: WebSocket, response: Response) {
        webSocket.send(config.serialize())
    }

    override fun onMessage(webSocket: WebSocket, text: String) {
        println("MESSAGE: $text")
    }

    override fun onMessage(webSocket: WebSocket, bytes: ByteString) {
        println("MESSAGE: " + bytes.hex())
    }

    override fun onClosing(webSocket: WebSocket, code: Int, reason: String) {
        webSocket.close(1000, null)
        Log.i(LogTag, "Closing bridge socket {code=$code, reason=$reason}")
        onComplete.run()
    }

    override fun onFailure(webSocket: WebSocket, t: Throwable, response: Response?) {
        Log.e(LogTag, t.toString(), t)
    }

    private fun getBridgeControlUrl(): String {
        return "$apiRoot/v0/users/id/$user_id/bridges/id/$bridge_id/communication"
    }
}